from django.urls import path
from .views import GetNiceAuthView, GetNiceAuthUrlView, VerifyNiceAuthView

urlpatterns = [
    path('', GetNiceAuthView.as_view(), name='get_nice_auth'),
    path('url/', GetNiceAuthUrlView.as_view(), name='get_nice_auth_url'),
    path('verify/', VerifyNiceAuthView.as_view(), name='verify_auth_result'),
]



# Generated by Django 4.2.13 on 2024-06-13 09:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('niceauth', '0002_niceauthrequest_authtype_niceauthrequest_popupyn_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='niceauthresult',
            old_name='result_data',
            new_name='result',
        ),
    ]



# niceauth/admin.py

from django.contrib import admin
from .models import NiceAuthRequest, NiceAuthResult


class NiceAuthResultInline(admin.StackedInline):
    model = NiceAuthResult
    can_delete = False
    verbose_name_plural = 'Nice Auth Results'
    readonly_fields = ('result', 'created_at', 'updated_at')


@admin.register(NiceAuthRequest)
class NiceAuthRequestAdmin(admin.ModelAdmin):
    list_display = ('request_no', 'created_at', 'updated_at', 'return_url', 'authtype', 'popupyn')
    search_fields = ('request_no', 'return_url', 'authtype', 'popupyn')
    list_filter = ('authtype', 'popupyn', 'created_at', 'updated_at')
    readonly_fields = ('created_at', 'updated_at')
    fieldsets = (
        (None, {
            'fields': ('request_no', 'enc_data', 'integrity_value', 'token_version_id', 'key', 'iv', 'return_url', 'authtype', 'popupyn')
        }),
        ('Timestamps', {
            'fields': ('created_at', 'updated_at'),
        }),
    )
    inlines = [NiceAuthResultInline]


@admin.register(NiceAuthResult)
class NiceAuthResultAdmin(admin.ModelAdmin):
    list_display = ('request', 'created_at', 'updated_at')
    search_fields = ('request__request_no',)
    readonly_fields = ('result', 'created_at', 'updated_at')
    fieldsets = (
        (None, {
            'fields': ('request', 'result')
        }),
        ('Timestamps', {
            'fields': ('created_at', 'updated_at'),
        }),
    )



# Generated by Django 4.2.13 on 2024-06-13 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('niceauth', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='niceauthrequest',
            name='authtype',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
        migrations.AddField(
            model_name='niceauthrequest',
            name='popupyn',
            field=models.CharField(blank=True, max_length=1, null=True),
        ),
        migrations.AddField(
            model_name='niceauthrequest',
            name='return_url',
            field=models.URLField(blank=True, null=True),
        ),
    ]



from setuptools import setup, find_packages

setup(
    name="django-nice-auth",
    version="0.1.1",
    packages=find_packages(),
    install_requires=[
        "django>=3.0",
        "requests",
        "pycryptodome",
        "nice_auth",
    ],
    author="RUNNERS",
    author_email="dev@runners.im",
    description="A Django app for NICE authentication",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/RUNNERS-IM/django-nice-auth",
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



# Generated by Django 4.2.13 on 2024-06-12 07:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='NiceAuthRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated At')),
                ('request_no', models.CharField(max_length=30, unique=True)),
                ('enc_data', models.TextField()),
                ('integrity_value', models.TextField()),
                ('token_version_id', models.CharField(max_length=100)),
                ('key', models.CharField(max_length=32)),
                ('iv', models.CharField(max_length=32)),
            ],
            options={
                'verbose_name': 'Nice Auth Request',
                'verbose_name_plural': 'Nice Auth Requests',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='NiceAuthResult',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated At')),
                ('result_data', models.JSONField()),
                ('request', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='niceauth.niceauthrequest')),
            ],
            options={
                'verbose_name': 'Nice Auth Result',
                'verbose_name_plural': 'Nice Auth Results',
                'ordering': ['-created_at'],
            },
        ),
    ]



# niceauth/models.py

from django.db import models

class BaseModel(models.Model):
    created_at = models.DateTimeField('Created At', auto_now_add=True, null=True)
    updated_at = models.DateTimeField('Updated At', auto_now=True, null=True)

    class Meta:
        abstract = True

    def __str__(self):
        return '{}({})'.format(self.title, self.id)


class NiceAuthRequest(BaseModel):
    request_no = models.CharField(max_length=30, unique=True)
    enc_data = models.TextField()
    integrity_value = models.TextField()
    token_version_id = models.CharField(max_length=100)
    key = models.CharField(max_length=32)
    iv = models.CharField(max_length=32)
    return_url = models.URLField(max_length=200, null=True, blank=True)
    authtype = models.CharField(max_length=20, null=True, blank=True)
    popupyn = models.CharField(max_length=1, null=True, blank=True)

    class Meta:
        verbose_name = 'Nice Auth Request'
        verbose_name_plural = 'Nice Auth Requests'
        ordering = ['-created_at']


class NiceAuthResult(BaseModel):
    request = models.OneToOneField(NiceAuthRequest, on_delete=models.CASCADE)
    result = models.JSONField()

    class Meta:
        verbose_name = 'Nice Auth Result'
        verbose_name_plural = 'Nice Auth Results'
        ordering = ['-created_at']



from django.conf import settings
from django.http import JsonResponse
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404
import json
from .models import NiceAuthRequest, NiceAuthResult
from nice_auth.services import NiceAuthService
from nice_auth.exceptions import NiceAuthException


class GetNiceAuthView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get(self, request):
        return self.handle_request(request)

    def post(self, request):
        return self.handle_request(request)

    def handle_request(self, request):
        try:
            return_url = request.GET.get('return_url') if request.method == 'GET' else request.POST.get('return_url')
            service = NiceAuthService(
                base_url=settings.NICE_AUTH_BASE_URL,
                client_id=settings.NICE_CLIENT_ID,
                client_secret=settings.NICE_CLIENT_SECRET,
                product_id=settings.NICE_PRODUCT_ID,
                return_url=return_url or settings.NICE_RETURN_URL,
                authtype=settings.NICE_AUTHTYPE,
                popupyn=settings.NICE_POPUPYN
            )
            auth_data = service.get_nice_auth()
            auth_request = NiceAuthRequest.objects.create(
                request_no=auth_data["requestno"],
                enc_data=auth_data["enc_data"],
                integrity_value=auth_data["integrity_value"],
                token_version_id=auth_data["token_version_id"],
                key=auth_data["key"],
                iv=auth_data["iv"],
                return_url=return_url  # Store return_url
            )
            return JsonResponse({
                'request_no': auth_request.request_no,
                'enc_data': auth_request.enc_data,
                'integrity_value': auth_request.integrity_value,
                'token_version_id': auth_request.token_version_id
            })
        except NiceAuthException as e:
            return JsonResponse({'error': str(e)}, status=400)
        except Exception as e:
            return JsonResponse({'error': 'An unexpected error occurred: ' + str(e)}, status=500)


class GetNiceAuthUrlView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get(self, request):
        return self.handle_request(request)

    def post(self, request):
        return self.handle_request(request)

    def handle_request(self, request):
        try:
            return_url = request.GET.get('return_url') if request.method == 'GET' else request.POST.get('return_url')
            service = NiceAuthService(
                base_url=settings.NICE_AUTH_BASE_URL,
                client_id=settings.NICE_CLIENT_ID,
                client_secret=settings.NICE_CLIENT_SECRET,
                product_id=settings.NICE_PRODUCT_ID,
                return_url=return_url or settings.NICE_RETURN_URL,
                authtype=settings.NICE_AUTHTYPE,
                popupyn=settings.NICE_POPUPYN
            )
            auth_data = service.get_nice_auth()
            auth_request = NiceAuthRequest.objects.create(
                request_no=auth_data["requestno"],
                enc_data=auth_data["enc_data"],
                integrity_value=auth_data["integrity_value"],
                token_version_id=auth_data["token_version_id"],
                key=auth_data["key"],
                iv=auth_data["iv"],
                return_url=return_url  # Store return_url
            )
            nice_url = f"https://nice.checkplus.co.kr/CheckPlusSafeModel/service.cb?m=service&token_version_id={auth_data['token_version_id']}&enc_data={auth_data['enc_data']}&integrity_value={auth_data['integrity_value']}"
            return JsonResponse({'nice_auth_url': nice_url})
        except NiceAuthException as e:
            return JsonResponse({'error': str(e)}, status=400)
        except Exception as e:
            return JsonResponse({'error': 'An unexpected error occurred: ' + str(e)}, status=500)


class VerifyNiceAuthView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get(self, request):
        return self.handle_request(request)

    def post(self, request):
        return self.handle_request(request)

    def handle_request(self, request):
        try:
            data = request.GET if request.method == 'GET' else request.POST
            enc_data = data.get('enc_data')
            token_version_id = data.get('token_version_id')
            integrity_value = data.get('integrity_value')

            auth_request = get_object_or_404(NiceAuthRequest, token_version_id=token_version_id, integrity_value=integrity_value)
            key = auth_request.key
            iv = auth_request.iv

            service = NiceAuthService(
                base_url=settings.NICE_AUTH_BASE_URL,
                client_id=settings.NICE_CLIENT_ID,
                client_secret=settings.NICE_CLIENT_SECRET,
                product_id=settings.NICE_PRODUCT_ID,
                return_url=auth_request.return_url,  # Use stored return_url
                authtype=settings.NICE_AUTHTYPE,
                popupyn=settings.NICE_POPUPYN
            )

            result_data = service.verify_auth_result(enc_data, key, iv)
            auth_result = NiceAuthResult.objects.create(
                request=auth_request,
                result=result_data
            )
            return JsonResponse(result_data)
        except NiceAuthException as e:
            return JsonResponse({'error': str(e)}, status=400)
        except Exception as e:
            return JsonResponse({'error': 'An unexpected error occurred: ' + str(e)}, status=500)
